/*
 * File: bsd/parmacs_decl.h
 * Description: miscellaneous variable declarations and function prototypes
 * Comments: 
 * Copyright: Department of Computer Architecture - Polytechnic University of
 *   Catalonia (UPC)
 * Created: 1999/06 ernest@ac.upc.es (E.Artiaga)
 * Contact: gsomk@ac.upc.es
 * Last update:
 * History:
 */
/*
 * Disclaimer and copying policy:
 *
 * The Polytechnic University of Catalonia (UPC) and the authors disclaim
 * all warranties with regard to this software, including all implied
 * warranties of merchantability and fitness.
 * In no event shall the Polytechnic University of Catalonia (UPC) or the
 * authors be liable for any special, indirect or consequential damages or
 * any damages whatsoever resulting from loss of use, data or profits,
 * whether in an action of contract, negligence or other tortius action,
 * arising out of or in connection with the use or performance of this
 * software.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice and disclaimer, and also this permission notice, appear
 * in all copies.
 *
 */

#ifndef _BSD_PARMACS_DECL_H_
#define _BSD_PARMACS_DECL_H_

/* Global parmacs control data */
#ifdef PARMACS_INTERNALS
extern parmacs_data_t _parmacs_data;
extern parmacs_configuration_t _parmacs_config;
extern parmacs_process_t _parmacs_self;
#endif /* PARMACS_INTERNALS */

/* inline functions */

#ifdef PARMACS_INTERNALS

#ifdef PARMACS_NO_MSEM

#define MSEM_LOCKED 1
#define MSEM_UNLOCKED 0
#define MSEM_IF_NOWAIT 1
#define MSEM_IF_WAITERS 1

#ifdef __sgi
#define parmacs_test_and_set _test_and_set
#endif /* __sgi */

#if defined(__GNUC__) && defined(__i386__)
extern __inline__ int p_test_and_set(volatile int *, int);
extern __inline__ int p_test_and_set(address, value)
volatile int *address;
int value;
{
  register int v = value;
  __asm__ volatile ("xchgl %0, %1": "=&r" (v), "=m" (*address):: "memory");
  return v;
}
#define parmacs_test_and_set p_test_and_set
#endif /* defined(__GNUC__) && defined(__i386__) */

#ifndef parmacs_test_and_set
extern int test_and_set(int *, int);
#define parmacs_test_and_set test_and_set
#endif /* parmacs_test_and_set */

msemaphore *msem_init(msemaphore *, int);
int msem_lock(msemaphore *, int);
int msem_unlock(msemaphore *, int);

#endif /* PARMACS_NO_MSEM */

/* 
 * Mutual exclusion - internal use only
 */
 
#ifdef __GNUC__

static __inline__ void _parmacs_sem_lock(msemaphore *);
static __inline__ void _parmacs_sem_lock(msemaphore *sem)
{
  msem_lock(sem, 0);
}

static __inline__ void _parmacs_sem_unlock(msemaphore *);
static __inline__ void _parmacs_sem_unlock(msemaphore *sem)
{
  msem_unlock(sem, 0);
}

static __inline__ void _parmacs_internal_lock(int);
static __inline__ void
_parmacs_internal_lock(int mutex)
{
  _parmacs_sem_lock(&(_parmacs_data->mutex[mutex]));
}

static __inline__ void _parmacs_internal_unlock(int);
static __inline__ void
_parmacs_internal_unlock(int mutex)
{
  _parmacs_sem_unlock(&(_parmacs_data->mutex[mutex]));
}

#else /* __GNUC__ */

#define _parmacs_sem_lock(__sem__)          msem_lock(__sem__, 0)
#define _parmacs_sem_unlock(__sem__)        msem_unlock(__sem__, 0)
#define _parmacs_internal_lock(__mutex__)     \
  _parmacs_sem_lock(&(_parmacs_data->mutex[__mutex__]))

#define _parmacs_internal_unlock(__mutex__)   \
  _parmacs_sem_unlock(&(_parmacs_data->mutex[__mutex__]))

#endif /* __GNUC__ */

#endif /* PARMACS_INTERNALS */


/*
 * Function prototypes
 */


void *_parmacs_local_alloc(int);
void  _parmacs_local_free(void *);

void  _parmacs_panic(char *);
void  _parmacs_exit(void);
void  _parmacs_yield(void);

#endif /* _BSD_PARMACS_DECL_H_ */
