/*
 * File: irix.sprocs/parmacs_decl.h
 * Description: miscellaneous variable declarations and function prototypes
 * Comments: 
 * Copyright: Department of Computer Architecture - Polytechnic University of
 *   Catalonia (UPC)
 * Created: 1999/06 ernest@ac.upc.es (E.Artiaga)
 * Contact: gsomk@ac.upc.es
 * Last update:
 * History:
 */
/*
 * Disclaimer and copying policy:
 *
 * The Polytechnic University of Catalonia (UPC) and the authors disclaim
 * all warranties with regard to this software, including all implied
 * warranties of merchantability and fitness.
 * In no event shall the Polytechnic University of Catalonia (UPC) or the
 * authors be liable for any special, indirect or consequential damages or
 * any damages whatsoever resulting from loss of use, data or profits,
 * whether in an action of contract, negligence or other tortius action,
 * arising out of or in connection with the use or performance of this
 * software.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice and disclaimer, and also this permission notice, appear
 * in all copies.
 *
 */

#ifndef _IRIX_SPROCS_PARMACS_DECL_H_
#define _IRIX_SPROCS_PARMACS_DECL_H_

/* Global parmacs control data */
#ifdef PARMACS_INTERNALS
extern parmacs_data_t _parmacs_data;
extern parmacs_configuration_t _parmacs_config;

#define _parmacs_self_ptr ((parmacs_process_t *)(&(PRDA->usr_prda)))
#define _parmacs_self (*(_parmacs_self_ptr))

#endif /* PARMACS_INTERNALS */

#ifdef PARMACS_INTERNALS

/* inline functions */

/* 
 * Mutual exclusion - internal use only
 */
 
#ifdef __GNUC__

static __inline__ void _parmacs_sem_lock(parmacs_spin_lock_t);
static __inline__ void _parmacs_sem_lock(parmacs_spin_lock_t spin_lock)
{
  register int countdown;
  register int spin;
  
  while(test_and_set((parmacs_spin_lock_data *)spin_lock, 1)) {
    countdown = spin = _parmacs_config->spin_count;
    while(*spin_lock) {
      if (countdown >= 0) {
        if (!countdown) {
          _parmacs_yield();
          countdown = spin;
        } else {
          countdown --;
        }
      }
    }
  }
}

static __inline__ void _parmacs_sem_unlock(parmacs_spin_lock_t);
static __inline__ void _parmacs_sem_unlock(parmacs_spin_lock_t spin_lock)
{
  *spin_lock = 0;
}

static __inline__ void _parmacs_internal_lock(int);
static __inline__ void
_parmacs_internal_lock(int mutex)
{
  _parmacs_sem_lock(&(_parmacs_data->mutex[mutex]));
}

static __inline__ void _parmacs_internal_unlock(int);
static __inline__ void
_parmacs_internal_unlock(int mutex)
{
  _parmacs_sem_unlock(&(_parmacs_data->mutex[mutex]));
}

#else /* __GNUC__ */

#define _parmacs_sem_lock(__spin_lock__)                            \
do {                                                                \
  register int __countdown__;                                       \
  register int __spin__;                                            \
                                                                    \
  while(test_and_set((parmacs_spin_lock_data *)__spin_lock__, 1)) { \
    __countdown__ = __spin__ = _parmacs_config->spin_count;         \
    while(*__spin_lock__) {                                         \
      if (__countdown__ >= 0) {                                     \
        if (!__countdown__) {                                       \
          _parmacs_yield();                                         \
          __countdown__ = __spin__;                                 \
        } else {                                                    \
          __countdown__ --;                                         \
        }                                                           \
      }                                                             \
    }                                                               \
  }                                                                 \
} while (0)

#define _parmacs_sem_unlock(__spin_lock__) ((*(__spin_lock__)) = 0)
#define _parmacs_internal_lock(__mutex__)     \
  _parmacs_sem_lock(&(_parmacs_data->mutex[__mutex__]))

#define _parmacs_internal_unlock(__mutex__)   \
  _parmacs_sem_unlock(&(_parmacs_data->mutex[__mutex__]))

#endif /* __GNUC__ */

#endif /* PARMACS_INTERNALS */


/*
 * Function prototypes
 */


void *_parmacs_local_alloc(int);
void  _parmacs_local_free(void *);

void  _parmacs_panic(char *);
void  _parmacs_exit(void);
void  _parmacs_yield(void);

#endif /* _IRIX_SPROCS_PARMACS_DECL_H_ */
