/*
 * File: lwp/parmacs_decl.h
 * Description: miscellaneous variable declarations and function prototypes
 * Comments: 
 * Copyright: Department of Computer Architecture - Polytechnic University of
 *   Catalonia (UPC)
 * Created: 1999/06 ernest@ac.upc.es (E.Artiaga)
 * Contact: gsomk@ac.upc.es
 * Last update:
 * History:
 */
/*
 * Disclaimer and copying policy:
 *
 * The Polytechnic University of Catalonia (UPC) and the authors disclaim
 * all warranties with regard to this software, including all implied
 * warranties of merchantability and fitness.
 * In no event shall the Polytechnic University of Catalonia (UPC) or the
 * authors be liable for any special, indirect or consequential damages or
 * any damages whatsoever resulting from loss of use, data or profits,
 * whether in an action of contract, negligence or other tortius action,
 * arising out of or in connection with the use or performance of this
 * software.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice and disclaimer, and also this permission notice, appear
 * in all copies.
 *
 */

#ifndef _LWP_PARMACS_DECL_H_
#define _LWP_PARMACS_DECL_H_

/* Global parmacs control data */
#ifdef PARMACS_INTERNALS
extern parmacs_data_t _parmacs_data;
extern parmacs_configuration_t _parmacs_config;

/* inline functions */

/* 
 * Mutual exclusion - internal use only
 */

#ifdef PARMACS_SPIN_LOCK
#ifdef __GNUC__
static __inline__ void _parmacs_sem_lock(lwp_mutex_t *);
static __inline__ void _parmacs_sem_lock(lwp_mutex_t *mutex)
{
  register int countdown;
  register int spin;
  
  countdown = spin = _parmacs_config->spin_count;
  while(_lwp_mutex_trylock(mutex)) {
    if (countdown >= 0) {
      if (!countdown) {
	_parmacs_yield();
	countdown = spin;
      } else {
	countdown --;
      }
    }
  }
}

#else /* __GNUC__ */

#define _parmacs_sem_lock(__mutex__)                                \
do {                                                                \
  register int __countdown__;                                       \
  register int __spin__;                                            \
                                                                    \
  __countdown__ = __spin__ = _parmacs_config->spin_count;           \
  while(_lwp_mutex_trylock(__mutex__)) {                            \
    if (__countdown__ >= 0) {                                       \
      if (!__countdown__) {                                         \
        _parmacs_yield();                                           \
        __countdown__ = __spin__;                                   \
      } else {                                                      \
        __countdown__ --;                                           \
      }                                                             \
    }                                                               \
  }                                                                 \
} while (0)

#endif /* _GNUC_ */

#else /* PARMACS_SPIN_LOCK */
#define _parmacs_sem_lock(x) _lwp_mutex_lock(x)
#endif /* PARMACS_SPIN_LOCK */

#define _parmacs_sem_unlock(x) _lwp_mutex_unlock(x)

#define _parmacs_internal_lock(__mutex__)     \
  _parmacs_sem_lock(&(_parmacs_data->mutex[__mutex__]))

#define _parmacs_internal_unlock(__mutex__)   \
  _parmacs_sem_unlock(&(_parmacs_data->mutex[__mutex__]))
  
#ifndef LWP_MUTEX_INITIALIZER
#define _lwp_mutex_init(x) memset((x), 0, sizeof(lwp_mutex_t))
#endif /* LWP_MUTEX_INITIALIZER */

#ifndef LWP_COND_INITIALIZER
#define _lwp_cond_init(x) memset((x), 0, sizeof(lwp_cond_t))
#endif /* LWP_COND_INITIALIZER */
  
#endif /* PARMACS_INTERNALS */

/*
 * Function prototypes
 */

void *_parmacs_local_alloc(int);
void  _parmacs_local_free(void *);

void  _parmacs_panic(char *);
void  _parmacs_exit(void);
void  _parmacs_yield(void);

#endif /* _LWP_PARMACS_DECL_H_ */
