/*
 * File: pthreads.sparc/parmacs_types.h
 * Description: miscellaneous type definitions
 * Comments: 
 * Copyright: Department of Computer Architecture - Polytechnic University of
 *   Catalonia (UPC)
 * Created: 1999/06 ernest@ac.upc.es (E.Artiaga)
 * Contact: gsomk@ac.upc.es
 * Last update:
 * History:
 */
/*
 * Disclaimer and copying policy:
 *
 * The Polytechnic University of Catalonia (UPC) and the authors disclaim
 * all warranties with regard to this software, including all implied
 * warranties of merchantability and fitness.
 * In no event shall the Polytechnic University of Catalonia (UPC) or the
 * authors be liable for any special, indirect or consequential damages or
 * any damages whatsoever resulting from loss of use, data or profits,
 * whether in an action of contract, negligence or other tortius action,
 * arising out of or in connection with the use or performance of this
 * software.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice and disclaimer, and also this permission notice, appear
 * in all copies.
 *
 */

#ifndef _PTHREADS_SPARC_PARMACS_TYPES_H_
#define _PTHREADS_SPARC_PARMACS_TYPES_H_

typedef volatile unsigned long parmacs_spin;
typedef parmacs_spin *parmacs_spin_t;
extern unsigned long int parmacs_tas(parmacs_spin_t, unsigned long int); 
#define PARMACS_TAS_FREE 0
#define PARMACS_TAS_BUSY 255
#define PARMACS_SPIN_LOCKED 1
#define PARMACS_SPIN_UNLOCKED 0

#ifdef PARMACS_INTERNALS
/* Parmacs configuration */
typedef struct parmacs_configuration {
  char              *config_file;
  int                busy_wait_count;
  int                spin_count;
  int                max_barriers;
  int                max_globals;
  int                max_events;
  int                max_locks;
  int                page_size;
  int                stack_size;
  unsigned long int  shared_memory;
  int                memory_alignment;
  int                max_processes;
  int                kernel_limit;
  int                timestamp;
} parmacs_configuration;

typedef parmacs_configuration *parmacs_configuration_t;

#endif /* PARMACS_INTERNALS */

/* Barrier structure */
typedef struct parmacs_barrier {
  parmacs_spin s;          /* spin lock for mutual exclusion */
  volatile int n;          /* number of threads waiting */
  volatile int seq;        /* sequence number for barrier operation */
  volatile int active;     /* barrier is being used */
} parmacs_barrier;

/* Global subscript structure */
typedef struct parmacs_global {
  parmacs_spin s;          /* spin lock for mutual exclusion */
  volatile int i;          /* next subscript */
  volatile int n;          /* number of processes waiting */
  volatile int seq;        /* sequence number for barrier operation */
  volatile int active;     /* barrier is being used */
} parmacs_global;

/* Pause/event structure */
typedef struct parmacs_event {
  parmacs_spin s;          /* spin lock for mutual exclusion */
  volatile int v;          /* event value */
  void *blocked;           /* processes waiting for event to be set */
} parmacs_event;

/* lock structure */
typedef struct parmacs_lock {
  parmacs_spin s;          /* spin lock */
} parmacs_lock;

/* Time stamps */
typedef unsigned long int parmacs_timestamp;

/* Processes */
typedef pthread_t parmacs_pid;

#ifdef PARMACS_INTERNALS
typedef struct parmacs_process {
  struct parmacs_process *link;  /* Queues */
  volatile parmacs_event *event; /* waiting for this event */
  int          barrier_seq;      /* sequence number of current barrier op. */
  int          global_seq;       /* sequence number of current glob. subs. op.*/
  volatile int            ref_count;/* Reference counter */
  pthread_mutex_t         m;        /* Mutual exclusion */
  pthread_cond_t          c;        /* condition variable - wait for children */
  volatile int            children; /* Number of child processes */
  volatile int            finished; /* Number of finished children */
  struct parmacs_process *parent;   /* Parent process */
  void         (*routine)();        /* code to execute */
} parmacs_process;

typedef parmacs_process *parmacs_process_t;

/* Parmacs control structure */
#define PARMACS_MAIN_ID    0
#define PARMACS_CONFIG_ID  1
#define PARMACS_MEMORY_ID  2
#define PARMACS_BARRIER_ID 3
#define PARMACS_GLOBAL_ID  4
#define PARMACS_EVENT_ID   5
#define PARMACS_LOCK_ID    6
#define PARMACS_PROCESS_ID 7
#define PARMACS_MAX_ID     8
typedef struct parmacs_data {
  parmacs_timestamp start_time;  /* Starting time */
  parmacs_timestamp end_time;    /* Ending time */
  pthread_key_t     private;     /* Private data area per pthread */
  parmacs_barrier  *barrier;     /* Barrier structures */
  int               n_barriers;
  parmacs_global   *global;      /* Global subscript structures */
  int               n_globals;
  parmacs_event    *event;       /* Event structures */
  int               n_events;
  parmacs_lock     *lock;        /* Lock structures */
  int               n_locks;
  parmacs_process  *process;     /* Parmacs processes */
  parmacs_process_t free_process;/* Free processes queue */
  parmacs_spin     *mutex;       /* Semaphores for mutual exclusions */
} parmacs_data;

typedef parmacs_data *parmacs_data_t;

#endif /* PARMACS_INTERNALS */

#endif /* _PTHREADS_SPARC_PARMACS_TYPES_H_ */
