/*
 * File: sysv/parmacs_decl.h
 * Description: miscellaneous variable declarations and function prototypes
 * Comments: 
 * Copyright: Department of Computer Architecture - Polytechnic University of
 *   Catalonia (UPC)
 * Created: 1999/06 ernest@ac.upc.es (E.Artiaga)
 * Contact: gsomk@ac.upc.es
 * Last update:
 * History:
 */
/*
 * Disclaimer and copying policy:
 *
 * The Polytechnic University of Catalonia (UPC) and the authors disclaim
 * all warranties with regard to this software, including all implied
 * warranties of merchantability and fitness.
 * In no event shall the Polytechnic University of Catalonia (UPC) or the
 * authors be liable for any special, indirect or consequential damages or
 * any damages whatsoever resulting from loss of use, data or profits,
 * whether in an action of contract, negligence or other tortius action,
 * arising out of or in connection with the use or performance of this
 * software.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice and disclaimer, and also this permission notice, appear
 * in all copies.
 *
 */

#ifndef _SYSV_PARMACS_DECL_H_
#define _SYSV_PARMACS_DECL_H_

/* Global parmacs control data */
extern parmacs_data_t _parmacs_data;
#ifdef PARMACS_INTERNALS
extern parmacs_configuration_t _parmacs_config;
#endif /* PARMACS_INTERNALS */

/* inline functions */

#ifdef PARMACS_INTERNALS
/* 
 * Mutual exclusion - internal use only
 */
 
#ifdef __GNUC__

static __inline__ void _parmacs_sem_lock(int, int);
static __inline__ void _parmacs_sem_lock(int set, int sem)
{
  struct sembuf operation;
  
  operation.sem_num = sem;
  operation.sem_op  = -1;
  operation.sem_flg = 0;
  semop(set, &operation, 1);
}

static __inline__ void _parmacs_sem_unlock(int, int);
static __inline__ void _parmacs_sem_unlock(int set, int sem)
{
  struct sembuf operation;

  operation.sem_num = sem;
  operation.sem_op  = 1;
  operation.sem_flg = 0;
  semop(set, &operation, 1);
}

static __inline__ void _parmacs_internal_lock(int);
static __inline__ void
_parmacs_internal_lock(int mutex)
{
  _parmacs_sem_lock(_parmacs_data->mutex, mutex);
}

static __inline__ void _parmacs_internal_unlock(int);
static __inline__ void
_parmacs_internal_unlock(int mutex)
{
  _parmacs_sem_unlock(_parmacs_data->mutex, mutex);
}

#else /* __GNUC__ */

#define _parmacs_sem_lock(__set__, __sem__)   \
do {                                          \
  struct sembuf __operation__;                \
                                              \
  __operation__.sem_num = __sem__;            \
  __operation__.sem_op  = -1;                 \
  __operation__.sem_flg = 0;                  \
  semop(__set__, &__operation__, 1);          \
} while(0)

#define _parmacs_sem_unlock(__set__, __sem__) \
do {                                          \
  struct sembuf __operation__;                \
                                              \
  __operation__.sem_num = __sem__;            \
  __operation__.sem_op  = 1;                  \
  __operation__.sem_flg = 0;                  \
  semop(__set__, &__operation__, 1);          \
} while(0)

#define _parmacs_internal_lock(__mutex__)     \
  _parmacs_sem_lock(_parmacs_data->mutex, __mutex__)

#define _parmacs_internal_unlock(__mutex__)   \
  _parmacs_sem_unlock(_parmacs_data->mutex, __mutex__)

#endif /* __GNUC__ */

#endif /* PARMACS_INTERNALS */


/*
 * Function prototypes
 */


void *_parmacs_local_alloc(int);
void  _parmacs_local_free(void *);

void  _parmacs_panic(char *);
void  _parmacs_exit(void);
void  _parmacs_yield(void);

#endif /* _SYSV_PARMACS_DECL_H_ */
