/*
 * File: sysv/parmacs_types.h
 * Description: miscellaneous type definitions
 * Comments: 
 * Copyright: Department of Computer Architecture - Polytechnic University of
 *   Catalonia (UPC)
 * Created: 1999/06 ernest@ac.upc.es (E.Artiaga)
 * Contact: gsomk@ac.upc.es
 * Last update:
 * History:
 */
/*
 * Disclaimer and copying policy:
 *
 * The Polytechnic University of Catalonia (UPC) and the authors disclaim
 * all warranties with regard to this software, including all implied
 * warranties of merchantability and fitness.
 * In no event shall the Polytechnic University of Catalonia (UPC) or the
 * authors be liable for any special, indirect or consequential damages or
 * any damages whatsoever resulting from loss of use, data or profits,
 * whether in an action of contract, negligence or other tortius action,
 * arising out of or in connection with the use or performance of this
 * software.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice and disclaimer, and also this permission notice, appear
 * in all copies.
 *
 */

#ifndef _SYSV_PARMACS_TYPES_H_
#define _SYSV_PARMACS_TYPES_H_

#ifdef PARMACS_INTERNALS
/* Parmacs configuration */
typedef struct parmacs_configuration {
  char              *config_file;
  int                busy_wait_count;
  int                max_barriers;
  int                max_globals;
  int                max_events;
  int                max_locks;
  int                page_size;
  int                stack_size;
  unsigned long int  shared_memory;
  int                memory_alignment;
  int                timestamp;
} parmacs_configuration;

typedef parmacs_configuration *parmacs_configuration_t;

/* Semun structure */
#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
/* union semun is defined by including <sys/sem.h> */
typedef union semun parmacs_semun;
#else
typedef union parmacs_semun {
  int val;
  struct semid_ds *buf;
  ushort *array;
} parmacs_semun;
#endif

#endif /* PARMACS_INTERNALS */

/* Barrier structure */
typedef struct parmacs_barrier {
  volatile int n; /* Number of processes blocked in barrier */
  int          s; /* Semaphore set with 2 entries: 0->mutex 1->blocking queue */
} parmacs_barrier;

/* Global subscript structure */
typedef struct parmacs_global {
  volatile int i; /* Next subscript */
  volatile int n; /* Number of processes finished */
  int          s; /* Semaphore set with 2 entries: 0->mutex 1->blocking queue */
} parmacs_global;

/* Pause/event structure */
typedef struct parmacs_event {
  int s; /* Semaphore set with 3 entries: 0->mutex 1->wait_clear 2->wait_set */
} parmacs_event;

/* lock structure */
typedef struct parmacs_lock {
  int s; /* Semaphore */
} parmacs_lock;

/* Time stamps */
typedef unsigned long int parmacs_timestamp;

/* Processes */
typedef unsigned int parmacs_pid;

/* Parmacs control structure */
#define PARMACS_MAIN_ID    0
#define PARMACS_CONFIG_ID  1
#define PARMACS_MEMORY_ID  2
#define PARMACS_BARRIER_ID 3
#define PARMACS_GLOBAL_ID  4
#define PARMACS_EVENT_ID   5
#define PARMACS_LOCK_ID    6
#define PARMACS_MAX_ID     7
typedef struct parmacs_data {
  parmacs_timestamp start_time;  /* Starting time */
  parmacs_timestamp end_time;    /* Ending time */
#ifdef PARMACS_NO_SCHED_YIELD
  int               blocking_fd; /* Blocking fd to implement processor yield */
#endif /* PARMACS_NO_SCHED_YIELD */  
  parmacs_barrier  *barrier;     /* Barrier structures */
  int               n_barriers;
  parmacs_global   *global;     /* Global subscript structures */
  int               n_globals;
  parmacs_event    *event;       /* Event structures */
  int               n_events;
  parmacs_lock     *lock;        /* Lock structures */
  int               n_locks;
  int               segment[PARMACS_MAX_ID]; /* Segment identifiers */
  int               mutex;       /* Semaphore set for mutual exclusions */
  void             *shm_address; /* Beginning of the shared memory segment */
  unsigned long     shm_size;    /* Size of the shared memory area */
  unsigned long     shm_avail;   /* Amount of available shared memory */
  void             *shm_pointer; /* Pointer to free memory */
} parmacs_data;

typedef parmacs_data *parmacs_data_t;

#endif /* _SYSV_PARMACS_TYPES_H_ */
